import { EventEmitter } from "./event-emitter";
import { nodeFactory } from "./nodes/node-factory";
export class Page extends EventEmitter {
    get documentRef() {
        return this.iframe.contentDocument || this.iframe.contentWindow?.document;
    }
    get windowRef() {
        return this.iframe.contentWindow;
    }
    constructor(container, title, schema, options, rootDocumentRef) {
        super();
        this.container = container;
        this.title = title;
        this.schema = schema;
        this.options = options;
        this.rootDocumentRef = rootDocumentRef;
        this.libraries = [];
        this.registerIframe();
        this.iframe.addEventListener('load', () => {
            this.registerStyleSheet();
            this.emit('ready');
            this.linkLibrary({ name: 'custom-styles', href: './lib-styles.css', rel: 'stylesheet' });
        });
    }
    getRootElement() {
        return this.rootElement;
    }
    getStyleSheet() {
        return this.styleSheet;
    }
    getLinkedLibraries() {
        return this.libraries.map(lib => ({
            ...Array.from(lib.attributes).reduce((r, attr) => (r[attr.name] = attr.value) && r, {}),
            name: lib.id
        }));
    }
    getSchema() {
        return this.rootElement.getSchema();
    }
    schemaChanged() {
        this.emit('schema:change', this.getSchema());
    }
    getOptions() {
        return this.options;
    }
    buildElements(container) {
        this.rootElement = nodeFactory(this.schema, this);
        container.appendChild(this.rootElement.nativeElement);
    }
    customElementFactory(element) {
        if (!this.options.customElementFactory) {
            console.error('Custom element factory was not provided');
            return this.getStubElement();
        }
        return this.options.customElementFactory(element);
    }
    nodeCreated(node) {
        this.emit('node:create', node);
    }
    findElement(predicate) {
        return this.rootElement.findElement(predicate);
    }
    linkLibrary({ name, href, rel, ...rest }) {
        const link = this.documentRef.createElement('link');
        link.href = href;
        link.rel = rel;
        link.id = name;
        Object.entries(rest).forEach(([key, value]) => {
            link.setAttribute(key, value);
        });
        this.documentRef.head.appendChild(link);
        return new Promise((res, rej) => {
            link.onload = () => {
                this.libraries.push(link);
                res();
            };
            link.onerror = () => {
                console.log('error linking lib');
                rej();
            };
        });
    }
    logError(node, errorMessage) {
        this.emit('node:error', { node, errorMessage });
        console.error(errorMessage);
    }
    registerIframe() {
        this.iframe = this.rootDocumentRef.createElement('iframe');
        this.iframe.classList.add('page-iframe');
        this.container.appendChild(this.iframe);
    }
    registerStyleSheet() {
        const styleSheet = this.documentRef.createElement('style');
        styleSheet.setAttribute('type', 'text/css');
        this.documentRef.head.appendChild(styleSheet);
        this.styleSheet = styleSheet.sheet;
    }
    getStubElement() {
        return this.documentRef.createElement('span');
    }
}
//# sourceMappingURL=page.js.map